package cn.iocoder.yudao.module.shipment.controller.admin;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.i18n.core.I18nMessage;
import cn.iocoder.yudao.framework.idempotent.core.annotation.Idempotent;
import cn.iocoder.yudao.framework.limiter.dynamic.DynamicRateLimiter;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import cn.iocoder.yudao.framework.redis.helper.RedisHelper;
import cn.iocoder.yudao.module.order.vo.order.OrderBackPageVO;
import cn.iocoder.yudao.module.order.vo.order.OrderMarkupVO;
import cn.iocoder.yudao.module.order.vo.order.OrderQueryVO;
import cn.iocoder.yudao.module.order.vo.orderSplit.OrderSplitBackVO;
import cn.iocoder.yudao.module.shipment.convert.BoxConvert;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxDO;
import cn.iocoder.yudao.module.shipment.service.box.BoxService;
import cn.iocoder.yudao.module.shipment.vo.box.*;
import cn.iocoder.yudao.module.shipment.vo.boxTally.BoxGuanlianOrderBackVO;
import cn.iocoder.yudao.module.shipment.vo.boxTally.BoxOrderLocationUpdateReq;
import cn.iocoder.yudao.module.shipment.vo.boxTally.BoxTallyBackVO;
import cn.iocoder.yudao.module.system.api.file.FileMakeApi;
import cn.iocoder.yudao.module.system.api.file.dto.FileMakeReqDTO;
import cn.iocoder.yudao.module.system.enums.download.DownloadTypeEnum;
import cn.iocoder.yudao.module.system.enums.sms.SmsNodeEnum;
import cn.iocoder.yudao.module.wealth.vo.receivable.ReceivableBoxReportQueryVO;
import com.alibaba.fastjson.JSONObject;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.text.MessageFormat;
import java.util.Collection;
import java.util.List;
import java.util.Objects;
import java.util.concurrent.TimeUnit;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.error;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.EXPORT;
import static cn.iocoder.yudao.module.order.enums.ErrorCodeConstants.PARAM_SHIPMENT_NOT_NULL;
import static cn.iocoder.yudao.module.shipment.controller.admin.constant.Constant.BOX_UPDATE_KEY;
import static cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.BOX_UPDATE_REPEAT_COMMIT;

@Validated
@RestController
@Api(tags = "管理后台 - 出货")
@RequestMapping("/shipment/box")
public class BoxController {
    @Value("${ue.templates-url}")
    private String path;

    @Resource
    private BoxService boxService;

    @Resource
    private RedisHelper redisHelper;

    @Resource
    private FileMakeApi fileMakeApi;

    @GetMapping("/checkOrderScheduling")
    @ApiImplicitParam(name = "orderId", value = "订单ID", required = true, example = "11", dataTypeClass = Long.class)
    public CommonResult<BoxBackVO> checkOrderScheduling(@RequestParam("orderId") Long orderId) {
        BoxDO boxDO = boxService.checkOrderScheduling(orderId);
        return success(BoxConvert.INSTANCE.convert(boxDO));
    }

    @PostMapping("/create")
    @ApiOperation("创建出货单，user this")
    @PreAuthorize("@ss.hasPermission('shipment:box:create')")
    @DynamicRateLimiter(base = "#Headers['Authorization']", permits = 1)
    @Idempotent(timeout = 5)
    public CommonResult<Long> createBox(@Valid @RequestBody BoxCreateReqVO createReqVO) {
        return success(boxService.createBox(createReqVO));
    }

    @PostMapping("/createAir")
    @ApiOperation("创建空运出货单，user this")
    @PreAuthorize("@ss.hasPermission('shipment:box:create:air')")
    @DynamicRateLimiter(base = "#Headers['Authorization']", permits = 1)
    @Idempotent(timeout = 5)
    public CommonResult<Long> createAirBox(@Valid @RequestBody BoxCreateReqVO createReqVO) {
        return success(boxService.createBox(createReqVO));
    }

    @PostMapping("/createSeaAir")
    @ApiOperation("创建海空空运出货单，user this")
    @PreAuthorize("@ss.hasPermission('shipment:box:create:seaAir')")
    @DynamicRateLimiter(base = "#Headers['Authorization']", permits = 1)
    @Idempotent(timeout = 5)
    public CommonResult<Long> createSeaAir(@Valid @RequestBody BoxCreateReqVO createReqVO) {
        return success(boxService.createBox(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新出货单，user this")
    @PreAuthorize("@ss.hasPermission('shipment:box:update')")
    @DynamicRateLimiter(base = "#Headers['Authorization']", permits = 1)
    @Idempotent(timeout = 5)
    public CommonResult<Boolean> updateBox(@Valid @RequestBody BoxUpdateReqVO updateReqVO) {
        String redisKey = MessageFormat.format(BOX_UPDATE_KEY, updateReqVO.getId().toString());
        Long count = redisHelper.incrBy(redisKey, 1);
        if (count > 1) {
            return error(BOX_UPDATE_REPEAT_COMMIT);
        }
        redisHelper.expire(redisKey, 1, TimeUnit.MINUTES);
        try {
            boxService.updateBox(updateReqVO);
        } catch (Exception e) {
            throw e;
        } finally {
            redisHelper.delete(redisKey);
        }
        return success(true);
    }

    @PostMapping("/agentSet")
    @ApiOperation("代理商设置，user this")
    @ApiImplicitParams({@ApiImplicitParam(name = "shipmentId", value = "出货单ID", required = true, example = "1024", dataTypeClass = Long.class),
            @ApiImplicitParam(name = "agentId", value = "代理商ID", required = true, example = "1024", dataTypeClass = Long.class)
    })
    @PreAuthorize("@ss.hasPermission('shipment:box:update')")
    public CommonResult<Boolean> agentSet(@RequestParam("shipmentId") Long shipmentId, @RequestParam("agentId") Long agentId) {
        boxService.updateAgent(shipmentId, agentId);
        return success(true);
    }

    @PostMapping("/shipConfigure")
    @ApiOperation("配船设置，user this")
    @ApiImplicitParams({@ApiImplicitParam(name = "shipmentId", value = "出货单ID", required = true, example = "1024", dataTypeClass = Long.class),
            @ApiImplicitParam(name = "saExmtStatus", value = "是否配船：1：免配，2：已配", required = true, example = "1", dataTypeClass = Integer.class),
            @ApiImplicitParam(name = "operateType", value = "操作类型，1-保存，2-提交", required = true, example = "1", dataTypeClass = Integer.class),
            @ApiImplicitParam(name = "configTime", value = "配船时间", required = false, example = "2022-08-02", dataTypeClass = String.class)
    })
    @PreAuthorize("@ss.hasPermission('shipment:box:update')")
    public CommonResult<Boolean> shipConfigure(@RequestParam("shipmentId") Long shipmentId, @RequestParam("saExmtStatus") Integer saExmtStatus,
                                               @RequestParam("operateType") Integer operateType, @RequestParam(name = "configTime", required = false) String configTime) {
        boxService.shipConfigure(shipmentId, saExmtStatus, operateType, configTime);
        return success(true);
    }

    @PostMapping("/noticeList")
    @ApiOperation("出货列表的弹窗提醒, use this")
    public CommonResult<List<BoxNoticeVO>> noticeList(@RequestBody BoxQueryVO boxQueryVO) {
        return success(boxService.noticeList(boxQueryVO));
    }

    @PostMapping("/airNoticeList")
    @ApiOperation("空运出货列表的弹窗提醒, use this")
    public CommonResult<List<BoxNoticeVO>> airNoticeList(@RequestBody BoxQueryVO boxQueryVO) {
        return success(boxService.airNoticeList(boxQueryVO));
    }

    @GetMapping("/generateBillLadingNo")
    @ApiImplicitParam(name = "shipmentId", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    public CommonResult<Boolean> generateBillLadingNo(@RequestParam("shipmentId") Long shipmentId) {
        boxService.generateBillLadingNo(shipmentId, true, false);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除出货单， use this")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('shipment:box:delete')")
    public CommonResult<Boolean> deleteBox(@RequestParam("id") Long id) {
        boxService.deleteBox(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得出货, use this")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
//    @PreAuthorize("@ss.hasPermission('shipment:box:query')")
    public CommonResult<BoxBackVO> getBoxDetail(@RequestParam("id") Long id) {
        //修改查询到港记录有多条记录报异常的BUG
        BoxBackVO boxBackVO = boxService.getBoxDetail(id);
        return success(boxBackVO);
    }

    @GetMapping("/list")
    @ApiOperation("获得出货列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
//    @PreAuthorize("@ss.hasPermission('shipment:box:query')")
    public CommonResult<List<BoxBackVO>> getBoxList(@RequestParam("ids") Collection<Long> ids) {
        List<BoxDO> list = boxService.getBoxList(ids);
        return success(BoxConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得出货分页, use this")
    //@PreAuthorize("@ss.hasPermission({'shipment:box:query', 'shipment:box:query:air'})")
    public CommonResult<PageResult<BoxBackVO>> getBoxPage(@Valid BoxQueryVO query, PageVO page) {

        return success(boxService.getBoxPage(query, page));
    }

    @PostMapping("/tallyList")
    @ApiImplicitParams({@ApiImplicitParam(name = "shipmentId", value = "出货单ID", required = true, example = "1024", dataTypeClass = Long.class)})
    @ApiOperation("理货列表， use this")
    public CommonResult<List<BoxTallyBackVO>> tallyList(@RequestParam("shipmentId") Long shipmentId) {
        return success(boxService.tallyList(shipmentId));
    }

    @PostMapping("/tallyListGuanlianOrder")
    @ApiImplicitParams({@ApiImplicitParam(name = "orderId", value = "订单id", required = true, example = "1024", dataTypeClass = Long.class)})
    @ApiOperation("理货-订单号-查看关联单， use this")
    public CommonResult<List<BoxGuanlianOrderBackVO>> tallyListGuanLianOrderList(@RequestParam("orderId") Long orderId) {
        return success(boxService.getGuanLianOrderList(orderId));
    }

    @PostMapping("/batchOrderLocationUpdate")
    @ApiOperation("批量修改储位, use this")
    public CommonResult<Boolean> batchOrderLocationUpdate(@RequestBody BoxOrderLocationUpdateReq req) {
        boxService.batchOrderLocationUpdate(req);
        return success(true);
    }

    @PostMapping("/tallyCommit")
    @ApiImplicitParams({@ApiImplicitParam(name = "shipmentId", value = "出货单ID", required = true, example = "1024", dataTypeClass = Long.class)})
    @ApiOperation("理货提交, use this")
    public CommonResult<Boolean> tallyCommit(@RequestParam("shipmentId") Long shipmentId) {
        boxService.tallyCommit(shipmentId);
        return success(true);
    }

    @PostMapping("/splitList")
    @ApiImplicitParams({@ApiImplicitParam(name = "orderId", value = "订单ID", required = true, example = "1024", dataTypeClass = Long.class),
            @ApiImplicitParam(name = "shipmentId", value = "出货单ID", required = true, example = "1024", dataTypeClass = Long.class)
    })
    @ApiOperation("获取拆单列表, use this")
    public CommonResult<List<OrderSplitBackVO>> splitList(@RequestParam("shipmentId") Long shipmentId, @RequestParam("orderId") Long orderId) {
        List<OrderSplitBackVO> orderSplitBackVOList = boxService.splitList(shipmentId, orderId);
        return success(orderSplitBackVOList);
    }


    @PostMapping("/part-refund/splitList")
    @ApiImplicitParams({@ApiImplicitParam(name = "orderId", value = "订单ID", required = true, example = "1024", dataTypeClass = Long.class),
            @ApiImplicitParam(name = "shipmentId", value = "出货单ID", required = true, example = "1024", dataTypeClass = Long.class)
    })
    @ApiOperation("获取部分退场拆单列表, use this")
    public CommonResult<List<OrderSplitBackVO>> partRefundSplitList(@RequestParam("shipmentId") Long shipmentId, @RequestParam("orderId") Long orderId) {
        List<OrderSplitBackVO> orderSplitBackVOList = boxService.partRefundSplitList(shipmentId, orderId);
        return success(orderSplitBackVOList);
    }

    @ApiOperation("更新异常订单状态, use this")
    @PostMapping("/updateAbnormalOrder")
    public CommonResult<Boolean> updateAbnormalOrder(@RequestBody OrderDealReqVO dealReqVO) {
        boxService.updateAbnormalOrder(dealReqVO);
        return success(true);
    }

    @ApiOperation("更新异常订单状态, use this")
    @PostMapping("/updateAbnormalOrderAir")
    public CommonResult<Boolean> updateAbnormalOrderAir(@RequestBody OrderDealReqVO dealReqVO) {
        boxService.updateAbnormalOrderAir(dealReqVO);
        return success(true);
    }

    @PostMapping("/customsOrderList")
    @ApiImplicitParams({@ApiImplicitParam(name = "customsTypes", value = "报关方式(多个以,连接),我司全代：1，自单代报：2，混合报关：3", example = "1024", dataTypeClass = String.class),
            @ApiImplicitParam(name = "shipmentId", value = "出货单ID", required = true, example = "1024", dataTypeClass = Long.class)
    })
    @ApiOperation("获取报关订单列表, use this")
    public CommonResult<List<OrderBackPageVO>> customsOrderList(@RequestParam("shipmentId") Long shipmentId, @RequestParam(value = "customsTypes", required = false) String customsTypes) {
        List<OrderBackPageVO> orderList = boxService.customsOrderList(shipmentId, customsTypes);
        return success(orderList);
    }

    @GetMapping("/shipmentOrderList")
    @ApiOperation("获取出货订单列表, use this")
    public CommonResult<List<OrderBackPageVO>> shipmentOrderList(@RequestParam("shipmentId") Long shipmentId) {
        List<OrderBackPageVO> orderList = boxService.customsOrderList(shipmentId, null);
        return success(orderList);
    }


    @GetMapping("/shipment/batch/markup/order/list")
    @ApiOperation("获得批量加价页面出货单订单列表")
    public CommonResult<List<OrderMarkupVO>> shipmentBatchMarkupOrderList(OrderQueryVO query) {
        if ((query.getShipmentId() == null || query.getShipmentId() == 0L) && StringUtils.isBlank(query.getContainerNumber())) {
            return error(PARAM_SHIPMENT_NOT_NULL);
        }
        if (StringUtils.isBlank(query.getContainerNumber())){
            BoxDO boxDO = boxService.getBox(query.getShipmentId());
            if (Objects.isNull(boxDO)){
                return error(PARAM_SHIPMENT_NOT_NULL);
            }
            query.setContainerNumber(boxDO.getSelfNo());
        }
        List<OrderMarkupVO> list = boxService.shipmentBatchMarkupOrderList(query);
        return success(list);
    }


    @GetMapping("/export-excel")
    @ApiOperation("导出出货 Excel")
    @PreAuthorize("@ss.hasPermission('shipment:box:export')")
    @OperateLog(type = EXPORT)
    public void exportBoxExcel(@Valid BoxQueryVO query,
                               HttpServletResponse response) throws IOException {
        List<BoxExportVO> list = boxService.getExportBoxList(query);
        // 导出 Excel
        ExcelUtils.write(response, "海运导出出货.xls", "shipment", BoxExportVO.class, list);
    }

    @GetMapping("/export-excel/seaAir")
    @ApiOperation("海空联运导出出货 Excel")
    @PreAuthorize("@ss.hasPermission('shipment:box:export:seaAir')")
    @OperateLog(type = EXPORT)
    public void exportBoxExcelSeaAir(@Valid BoxQueryVO query,
                                     HttpServletResponse response) throws IOException {
        List<BoxExportVO> list = boxService.getExportBoxList(query);
        // 导出 Excel
        ExcelUtils.write(response, "海空导出出货.xls", "shipment", BoxExportVO.class, list);
    }

    @GetMapping("/export-excel/air")
    @ApiOperation("空运导出出货 Excel")
    @PreAuthorize("@ss.hasPermission('shipment:box:export:air')")
    @OperateLog(type = EXPORT)
    public void exportBoxExcelAir(@Valid BoxQueryVO query,
                                  HttpServletResponse response) throws IOException {
        List<BoxExportVO> list = boxService.getExportBoxList(query);
        // 导出 Excel
        ExcelUtils.write(response, "空运导出出货.xls", "shipment", BoxExportVO.class, list);
    }


    @PostMapping("/shipment/sms-send")
    @ApiOperation("手动发送出货短信")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "shipmentId", value = "出货单ID", required = true, example = "1024", dataTypeClass = Long.class),
            @ApiImplicitParam(name = "scene", value = "场景编号", required = true, example = "20", dataTypeClass = String.class)
    })
    public CommonResult<Boolean> shipmentSmsSend(@RequestParam("shipmentId") Long shipmentId, @RequestParam("scene") String scene) {
        boxService.sendSms(shipmentId, SmsNodeEnum.getCodeByNodeValue(scene));
        return success(true);
    }


    @GetMapping("/dealCustomsSplitNotify")
    @ApiImplicitParam(name = "notifyId", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    public CommonResult<Boolean> dealCustomsSplitNotify(@RequestParam("notifyId") Long notifyId) {
        boxService.dealCustomsSplitNotify(notifyId);
        return success(true);
    }

    @GetMapping("/pageSettlement")
    @ApiOperation("自编号结算分页, use this")
//    @PreAuthorize("@ss.hasPermission('shipment:box:query')")
    public CommonResult<PageResult<BoxFinanceBackVO>> getBoxSettlementPage(@Valid BoxSettlementQueryVO query, PageVO page) {
        return success(boxService.getBoxSettlementPage(query, page));
    }

    //导出
    @GetMapping("/export-shipment-summary")
    @ApiOperation("导出自编号汇总列表 Excel")
    @OperateLog(type = EXPORT)
    public CommonResult exportOrderSummary(BoxSettlementQueryVO query, HttpServletResponse response) throws Exception {
        FileMakeReqDTO reqDTO = new FileMakeReqDTO();
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("query", JSONObject.toJSONString(query));
        reqDTO.setType(DownloadTypeEnum.SHIPMENT_SUMMERY_EXCEL_EXPORT.getType());
        reqDTO.setName("出货应收报表");
        reqDTO.setFileSuffix("xlsx");
        reqDTO.setTemporaryFile(true);
        reqDTO.setUserType(2);
        reqDTO.setLang(I18nMessage.getLang());
        reqDTO.setRequestParams(jsonObject.toJSONString());
        fileMakeApi.sendFileMake(reqDTO);
        return success("");
    }

    @GetMapping("/export-shipment-summary/new")
    @ApiOperation("导出自编号汇总列表 Excel")
    @OperateLog(type = EXPORT)
    public CommonResult exportOrderSummaryNew(ReceivableBoxReportQueryVO query, HttpServletResponse response) throws Exception {
        FileMakeReqDTO reqDTO = new FileMakeReqDTO();
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("query", JSONObject.toJSONString(query));
        reqDTO.setType(DownloadTypeEnum.SHIPMENT_SUMMERY_EXCEL_EXPORT_NEW.getType());
        reqDTO.setName("新出货应收报表");
        reqDTO.setFileSuffix("xlsx");
        reqDTO.setTemporaryFile(true);
        reqDTO.setUserType(2);
        reqDTO.setLang(I18nMessage.getLang());
        reqDTO.setRequestParams(jsonObject.toJSONString());
        fileMakeApi.sendFileMake(reqDTO);
        return success("");
    }
}
